const rnd = () => {
    return 1 - Math.random() * 2;
};

const vary = (base, range) => {
    return base + base * rnd() * range;
};

class CircleScene
{
    constructor() {

        this.config = {
            types: {
                micro:  {color: 300, count: 1300, size: 0.02, blur: 0.005, mv: 0.0001},
                small:  {color: 300, count: 200,  size: 0.04, blur: 0.001, mv: 0.0002},
                middle: {color: 300, count: 70,   size: 0.12, blur: 0.01,  mv: 0.0003},
                big:    {color: 300, count: 9,    size: 0.3,  blur: 0.1,   mv: 0.001},
                huge:   {color: 300, count: 3,    size: 0.6,  blur: 0.4,   mv: 0.001}
            }
        };

        this.width   = window.innerWidth;
        this.height  = window.innerHeight;
        this.refSize = (this.width + this.height) / 2;

        this.canvas = document.querySelector('canvas');
        this.engine = this.canvas.getContext('2d');
      
        this.xMove = 20;
      
        this.canvas.addEventListener('click', () => {
          this.xMove += 20;
        });

        this.canvas.setAttribute('width', this.width);
        this.canvas.setAttribute('height', this.height);

        this.objects = [];

        this.build();

    }

    build() {

        const color = (c) => { return `hsl(${c % 360 | 0}, 100%, 50%)`; };

        Object.keys(this.config.types).forEach(type => {

            const ref = this.config.types[type];

            for (let i = 0; i < ref.count; i++) {

                const size     = vary(ref.size, 0.5);
                const blur     = size / ref.size * ref.blur;
                const fullSize = size + blur * 2;
                const mid      = fullSize / 2;

                const canvas = document.createElement('canvas');
                const engine = canvas.getContext('2d');

                canvas.setAttribute('width', fullSize * this.refSize | 0);
                canvas.setAttribute('height', fullSize * this.refSize | 0);

                engine.beginPath();

                engine.fillStyle   = color(ref.color + Math.random() * 50);
                engine.globalAlpha = 0.1 + Math.random() * 0.4;

                const max = 2;
                for (let i = 0; i < max; i++) {

                    const blurPX = blur / ((i + 1) / max) * this.refSize / 2 | 0;

                    if (blurPX > 0) {
                        engine.filter = `blur(${blurPX}px)`;
                    }

                    engine.moveTo(this.refSize * mid, this.refSize * mid);
                    engine.arc(this.refSize * mid, this.refSize * mid, this.refSize * size / 2 * (i + 1) / max, 0, Math.PI * 2);
                    engine.fill();
                    engine.closePath();
                }

                const x = Math.random() * 100 * this.width;
                const y = Math.random() * 100 * this.height;

                const mx = vary(ref.mv, 0.5) * this.refSize * 3;
                const my = vary(ref.mv, 0.5) * this.refSize * 3;

                this.objects.push({
                    size: fullSize * this.refSize,
                    x, y,
                    mx, my,
                    canvas,
                    ref
                });

            }

        });

    }

    calc() {

        this.xMove = Math.max(1, this.xMove * 0.99);
      
        this.objects.forEach((obj) => {

            obj.x += obj.mx * this.xMove;
            obj.y += obj.my * this.xMove;

        });

        requestAnimationFrame(this.calc.bind(this));

    }

    draw() {

        this.engine.globalCompositeOperation = 'lighter';
        this.engine.clearRect(0, 0, this.width, this.height);

        this.objects.forEach((obj) => {

            const fx = (obj.x % (this.width  + obj.size)) - obj.size;
            const fy = this.height - (obj.y % (this.height + obj.size));

            this.engine.drawImage(
                obj.canvas,
                fx | 0,
                fy | 0
            );

        });

        requestAnimationFrame(this.draw.bind(this));

    }

    run() {

        this.calc();
        this.draw();

    }
}

new CircleScene().run();